
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#pragma once

#include <glib-object.h>
#include "wp/defs.h"

G_BEGIN_DECLS

/* enumerations from "base-dirs.h" */

WP_API
GType wp_base_dirs_flags_get_type (void);
#define WP_TYPE_BASE_DIRS_FLAGS (wp_base_dirs_flags_get_type())

/* enumerations from "core.h" */

WP_API
GType wp_core_features_get_type (void);
#define WP_TYPE_CORE_FEATURES (wp_core_features_get_type())

/* enumerations from "device.h" */

WP_API
GType wp_spa_device_features_get_type (void);
#define WP_TYPE_SPA_DEVICE_FEATURES (wp_spa_device_features_get_type())

/* enumerations from "dynamic-rules.h" */

WP_API
GType wp_dynamic_rules_features_get_type (void);
#define WP_TYPE_DYNAMIC_RULES_FEATURES (wp_dynamic_rules_features_get_type())

/* enumerations from "error.h" */

WP_API
GType wp_library_error_enum_get_type (void);
#define WP_TYPE_LIBRARY_ERROR_ENUM (wp_library_error_enum_get_type())

/* enumerations from "link.h" */

WP_API
GType wp_link_state_get_type (void);
#define WP_TYPE_LINK_STATE (wp_link_state_get_type())

/* enumerations from "log.h" */

WP_API
GType wp_log_topic_flags_get_type (void);
#define WP_TYPE_LOG_TOPIC_FLAGS (wp_log_topic_flags_get_type())

/* enumerations from "metadata.h" */

WP_API
GType wp_metadata_features_get_type (void);
#define WP_TYPE_METADATA_FEATURES (wp_metadata_features_get_type())

/* enumerations from "node.h" */

WP_API
GType wp_node_state_get_type (void);
#define WP_TYPE_NODE_STATE (wp_node_state_get_type())

WP_API
GType wp_node_features_get_type (void);
#define WP_TYPE_NODE_FEATURES (wp_node_features_get_type())

/* enumerations from "object-interest.h" */

WP_API
GType wp_constraint_type_get_type (void);
#define WP_TYPE_CONSTRAINT_TYPE (wp_constraint_type_get_type())

WP_API
GType wp_constraint_verb_get_type (void);
#define WP_TYPE_CONSTRAINT_VERB (wp_constraint_verb_get_type())

WP_API
GType wp_interest_match_get_type (void);
#define WP_TYPE_INTEREST_MATCH (wp_interest_match_get_type())

WP_API
GType wp_interest_match_flags_get_type (void);
#define WP_TYPE_INTEREST_MATCH_FLAGS (wp_interest_match_flags_get_type())

/* enumerations from "permission-manager.h" */

WP_API
GType wp_permission_manager_features_get_type (void);
#define WP_TYPE_PERMISSION_MANAGER_FEATURES (wp_permission_manager_features_get_type())

/* enumerations from "plugin.h" */

WP_API
GType wp_plugin_features_get_type (void);
#define WP_TYPE_PLUGIN_FEATURES (wp_plugin_features_get_type())

/* enumerations from "port.h" */

WP_API
GType wp_direction_get_type (void);
#define WP_TYPE_DIRECTION (wp_direction_get_type())

/* enumerations from "proxy.h" */

WP_API
GType wp_proxy_features_get_type (void);
#define WP_TYPE_PROXY_FEATURES (wp_proxy_features_get_type())

/* enumerations from "session-item.h" */

WP_API
GType wp_session_item_features_get_type (void);
#define WP_TYPE_SESSION_ITEM_FEATURES (wp_session_item_features_get_type())

/* enumerations from "settings.h" */

WP_API
GType wp_settings_spec_type_get_type (void);
#define WP_TYPE_SETTINGS_SPEC_TYPE (wp_settings_spec_type_get_type())

WP_API
GType wp_settings_features_get_type (void);
#define WP_TYPE_SETTINGS_FEATURES (wp_settings_features_get_type())

/* enumerations from "si-interfaces.h" */

WP_API
GType wp_si_adapter_ports_state_get_type (void);
#define WP_TYPE_SI_ADAPTER_PORTS_STATE (wp_si_adapter_ports_state_get_type())

/* enumerations from "state.h" */

WP_API
GType wp_state_metadata_features_get_type (void);
#define WP_TYPE_STATE_METADATA_FEATURES (wp_state_metadata_features_get_type())

/* enumerations from "transition.h" */

WP_API
GType wp_transition_step_get_type (void);
#define WP_TYPE_TRANSITION_STEP (wp_transition_step_get_type())

/* enumerations from "wp.h" */

WP_API
GType wp_init_flags_get_type (void);
#define WP_TYPE_INIT_FLAGS (wp_init_flags_get_type())

G_END_DECLS

/* Generated data ends here */

